<?php

namespace Drupal\inxmail_xml\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\inxmail_xml\Entity\TemplateEntity;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Symfony\Component\HttpFoundation\Response;

/**
* Class InxmailXmlController.
 *
 * @package Drupal\
*/
class InxmailXmlController extends ControllerBase {

  /**
  * Returns a XML response.
  */
  public function xml() {
    $current_path = \Drupal::service('path.current')->getPath();
    $templateEntityId = basename($current_path, '.xml');
    $templateEntity = TemplateEntity::load($templateEntityId);

    // $config = \Drupal::configFactory()->getEditable('inxmail_xml.settings');

    $xmlContent = '';
    $splittedTemplate = preg_split("/({{.*?}})/", $templateEntity->template, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

    foreach ($splittedTemplate as $templatesFragment) {
      if (str_starts_with($templatesFragment, '{{')) {
        $xmlContent .= $this->getDynamicContent($templatesFragment);
      } else {
        $xmlContent .= $templatesFragment;
      }
    }

    $response = new Response();
    $response->headers->set('Content-Type', 'text/xml; charset=utf-8');
    $response->setContent('<?xml version="1.0" encoding="UTF-8"?>' .
      $xmlContent
    );
    return $response;
  }

  /**
   * Get dynamic content of a fragment of the template.
   *
   * @param string $fragment
   *   Fragment of the template. The fragment has to be in the format:
   *   {{'key':'value','key':'value','key':'value'}}
   *
   * @return string
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  private function getDynamicContent($fragment) {
    // Transform the fragment to object.
    $fragment = str_replace('{{', '{', $fragment);
    $fragment = str_replace('}}', '}', $fragment);
    $fragmentObject = json_decode($fragment);

    if (property_exists($fragmentObject, 'task')) {
      switch ($fragmentObject->task) {
        case 'list_of_nodes':
          $contentType = $fragmentObject->content_type;
          return $this->getListOfNodes($contentType);
      }
    }
    return '';
  }

  /**
   * Get list of nodes.
   *
   * @param string $content_type
   *   Machine name of the content type.
   *
   * @return string
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  private function getListOfNodes($content_type) {
    $nids = \Drupal::entityQuery('node')
      ->condition('status', NodeInterface::PUBLISHED)
      ->condition('type', $content_type)
      ->sort('created', 'DESC')
      ->range(0, 10)
      ->execute();

    $nodes =  Node::loadMultiple($nids);

    $paragraphAd = '';
    foreach ($nodes as $node) {
      $paragraphAd .= '%%lt%%a href="' .
        $node->toUrl()->setAbsolute()->toString() . '"/%%gt%%' .
        $node->getTitle() . '%%lt%%/a%%gt%%%%lt%%br%%gt%%';

    }
    return htmlspecialchars($paragraphAd);
  }

}
